/**
 * 
 */
package com.up.sell.util;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author mingguangz yyyy-MM-dd HH:mm:ss
 */
public final class DateUtils {
	private static final Logger logger = LoggerFactory
			.getLogger(DateUtils.class);
	private static final TimeZone defaultTimeZone = TimeZone.getDefault();

	private DateUtils() {
	}

	/**
	 * 
	 * @return The current date time
	 */
	public static Date now() {
		return new Date();
	}

	/**
	 * 
	 * @param pattern
	 * @return The current date time
	 */
	public static String now(String pattern) {
		return format(now(), pattern);
	}

	/**
	 * 
	 * @param pattern
	 * @param timeZone
	 * @return The current date time
	 */
	public static String now(String pattern, TimeZone timeZone) {
		return format(now(), pattern, timeZone);
	}

	/**
	 * Adds a number of years to a date returning a new object. The original
	 * date object is unchanged.
	 * 
	 * @param date
	 *            the date, not null
	 * @param amount
	 *            the amount to add, may be negative
	 * @return the new date object with the amount added
	 */
	public static Date addYears(Date date, int amount) {
		return add(date, Calendar.YEAR, amount);
	}

	/**
	 * Adds a number of months to a date returning a new object. The original
	 * date object is unchanged.
	 * 
	 * @param date
	 *            the date, not null
	 * @param amount
	 *            the amount to add, may be negative
	 * @return the new date object with the amount added
	 */
	public static Date addMonths(Date date, int amount) {
		return add(date, Calendar.MONTH, amount);
	}

	/**
	 * Adds a number of weeks to a date returning a new object. The original
	 * date object is unchanged.
	 * 
	 * @param date
	 *            the date, not null
	 * @param amount
	 *            the amount to add, may be negative
	 * @return the new date object with the amount added
	 */
	public static Date addWeeks(Date date, int amount) {
		return add(date, Calendar.WEEK_OF_YEAR, amount);
	}

	/**
	 * Adds a number of days to a date returning a new object. The original date
	 * object is unchanged.
	 * 
	 * @param date
	 *            the date, not null
	 * @param amount
	 *            the amount to add, may be negative
	 * @return the new date object with the amount added
	 */
	public static Date addDays(Date date, int amount) {
		return add(date, Calendar.DAY_OF_MONTH, amount);
	}

	/**
	 * Adds a number of hours to a date returning a new object. The original
	 * date object is unchanged.
	 * 
	 * @param date
	 *            the date, not null
	 * @param amount
	 *            the amount to add, may be negative
	 * @return the new date object with the amount added
	 */
	public static Date addHours(Date date, int amount) {
		return add(date, Calendar.HOUR_OF_DAY, amount);
	}

	/**
	 * Adds a number of minutes to a date returning a new object. The original
	 * date object is unchanged.
	 * 
	 * @param date
	 *            the date, not null
	 * @param amount
	 *            the amount to add, may be negative
	 * @return the new date object with the amount added
	 */
	public static Date addMinutes(Date date, int amount) {
		return add(date, Calendar.MINUTE, amount);
	}

	/**
	 * Adds a number of seconds to a date returning a new object. The original
	 * date object is unchanged.
	 * 
	 * @param date
	 *            the date, not null
	 * @param amount
	 *            the amount to add, may be negative
	 * @return the new date object with the amount added
	 */
	public static Date addSeconds(Date date, int amount) {
		return add(date, Calendar.SECOND, amount);
	}

	/**
	 * Adds a number of milliseconds to a date returning a new object. The
	 * original date object is unchanged.
	 * 
	 * @param date
	 *            the date, not null
	 * @param amount
	 *            the amount to add, may be negative
	 * @return the new date object with the amount added
	 */
	public static Date addMilliseconds(Date date, int amount) {
		return add(date, Calendar.MILLISECOND, amount);
	}

	/**
	 * Adds to a date returning a new object. The original date object is
	 * unchanged.
	 * 
	 * @param date
	 *            the date, not null
	 * @param calendarField
	 *            the calendar field to add to
	 * @param amount
	 *            the amount to add, may be negative
	 * @return the new date object with the amount added
	 */
	private static Date add(Date date, int calendarField, int amount) {
		if (date == null) {
			throw new IllegalArgumentException("The date must not be null");
		}
		Calendar c = Calendar.getInstance();
		c.setTime(date);
		c.add(calendarField, amount);
		return c.getTime();
	}

	/**
	 * <p>
	 * Formats a date/time into a specific pattern.
	 * </p>
	 * 
	 * @param date
	 *            the date to format
	 * @param pattern
	 *            the pattern to use to format the date
	 *            如:"yyyy/MM/dd","yyyy-MM-dd"
	 *            ,"yyyyMMdd","yyyy年MM月dd日 HH时mm分ss秒", "yyyy-MM-dd HH:mm:ss"等等
	 * @return the formatted date
	 */
	public static String format(Date date, String pattern) {
		return format(date, pattern, defaultTimeZone);
	}

	/**
	 * 
	 * @param date
	 * @param pattern
	 * @return A Date parsed from the string.
	 */
	public static Date format(String date, String pattern) {
		return format(date, pattern, defaultTimeZone);
	}

	/**
	 * 
	 * @param date
	 * @param currentPattern
	 * @param targetPattern
	 * @return the formatted date
	 */
	public static String format(String date, String currentPattern,
			String targetPattern) {
		return format(date, currentPattern, targetPattern, defaultTimeZone);
	}

	/**
	 * <p>
	 * Formats a date/time into a specific pattern in a time zone.
	 * </p>
	 * 
	 * @param date
	 *            the date to format
	 * @param pattern
	 *            the pattern to use to format the date
	 * @param timeZone
	 *            the time zone to use, may be <code>null</code>
	 * @return the formatted date
	 */
	public static String format(Date date, String pattern, TimeZone timeZone) {
		SimpleDateFormat simpleDateFormat = new SimpleDateFormat(pattern);
		if (timeZone != null)
			simpleDateFormat.setTimeZone(timeZone);
		return simpleDateFormat.format(date);
	}

	/**
	 * 
	 * @param date
	 * @param pattern
	 * @param timeZone
	 * @return A Date parsed from the string.
	 */
	public static Date format(String date, String pattern, TimeZone timeZone) {
		SimpleDateFormat simpleDateFormat = new SimpleDateFormat(pattern);
		if (timeZone != null)
			simpleDateFormat.setTimeZone(timeZone);
		try {
			return simpleDateFormat.parse(date);
		} catch (ParseException e) {
			// e.printStackTrace();
			logger.error(e.getMessage(), e);
		}
		return null;
	}

	/**
	 * 
	 * @param date
	 * @param currentPattern
	 * @param targetPattern
	 * @param timeZone
	 * @return
	 */
	public static String format(String date, String currentPattern,
			String targetPattern, TimeZone timeZone) {
		SimpleDateFormat simpleDateFormat = new SimpleDateFormat(currentPattern);
		if (timeZone != null)
			simpleDateFormat.setTimeZone(timeZone);
		try {
			return format(simpleDateFormat.parse(date), targetPattern, timeZone);
		} catch (ParseException e) {
			logger.error(e.getMessage(), e);
		}
		return null;
	}

	/**
	 * 取得给定日期在当前年中的第几个星期
	 * 
	 * @param date
	 * @return
	 */
	public static int getWeekOfYear(Date date) {
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		return cal.get(Calendar.WEEK_OF_YEAR);
	}

	/**
	 * 取得给定日期是当前月中的第几个星期
	 * 
	 * @param date
	 * @return
	 */
	public static int getWeekOfMonth(Date date) {
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		return cal.get(Calendar.WEEK_OF_MONTH);
	}

	/**
	 * 取得给定日期是星期几
	 * 
	 * @param date
	 * @return 1~7
	 */
	public static String getWeek(Date date) {
		String[] weeks = { "星期日", "星期一", "星期二", "星期三", "星期四", "星期五", "星期六" };
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		int week_index = cal.get(Calendar.DAY_OF_WEEK);
		return weeks[week_index - 1];
	}

	/**
	 * 取得一年内有多少个星期
	 * 
	 * @param year
	 * @return
	 */
	public static int getWeeks(int year) throws ParseException {
		Calendar cal = Calendar.getInstance();
		cal.setTime(format(year + "-12-31", "yyyy-MM-dd"));
		return cal.get(Calendar.WEEK_OF_YEAR);
	}

	/**
	 * 计算出给定两个日期的时间间隔
	 * 
	 * @param startdate
	 * @param enddate
	 * @param pattern
	 *            时间间隔的单位("d"天 ，"h"小时，"m"分，"s"秒，"ms"豪秒)
	 * @return
	 */
	public static long dateDifference(Date startdate, Date enddate,
			String pattern) {
		long diffMillis = enddate.getTime() - startdate.getTime();
		if ("ms".equals(pattern))
			return diffMillis;
		else if ("s".equals(pattern))
			return diffMillis / 1000;
		else if ("m".equals(pattern))
			return diffMillis / (1000 * 60);
		else if ("h".equals(pattern))
			return diffMillis / 0x36ee80;
		else if ("d".equals(pattern))
			return diffMillis / 0x5265c00;
		else
			return 0;
	}

	/**
	 * 得到某个日期当月的天数
	 * 
	 * @param year
	 * @param month
	 * @return
	 */
	public static int getDaysOfMonth(Date date) {
		Calendar cal = Calendar.getInstance();
		cal.setTime(getLastDayOfMonth(date));
		return cal.get(Calendar.DATE);
	}

	/**
	 * 得到某个日期的当月最后一天
	 * 
	 * @param date
	 * @return
	 */
	public static Date getLastDayOfMonth(Date date) {
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		return addDays(addMonths(addDays(date, 1 - cal.get(Calendar.DATE)), 1),
				-1);
	}

	public static int getAgeByBirthday(Date birthday) {
		return getAgeByBirthday(birthday, new Date());
	}

	public static int getAgeByBirthday(Date birthday, Date endDate) {
		Calendar cal = Calendar.getInstance();
		cal.setTime(endDate);
		if (cal.before(birthday)) {
			throw new IllegalArgumentException(
					"The birthDay is before Now.It's unbelievable!");
		}
		int yearNow = cal.get(Calendar.YEAR);
		int monthNow = cal.get(Calendar.MONTH) + 1;
		int dayOfMonthNow = cal.get(Calendar.DAY_OF_MONTH);

		cal.setTime(birthday);
		int yearBirth = cal.get(Calendar.YEAR);
		int monthBirth = cal.get(Calendar.MONTH) + 1;
		int dayOfMonthBirth = cal.get(Calendar.DAY_OF_MONTH);

		int age = yearNow - yearBirth;

		if (monthNow <= monthBirth) {
			if (monthNow == monthBirth) {
				if (dayOfMonthNow < dayOfMonthBirth) {
					age--;
				}
			} else {
				age--;
			}
		}
		return age;
	}
	
	public static Date beginTimeHelper(Date date) {
		try {
			if (null != date) {
				SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd hh:mm:ss");
				SimpleDateFormat sdfShort = new SimpleDateFormat("yyyy/MM/dd");
				return sdf.parse(sdfShort.format(date) + " 00:00:00");
			}
		} catch (ParseException e) {
			e.printStackTrace();
		}
		return date;
	}
	
	public static Date endTimeHelper(Date date) {
		try {
			if (null != date) {
				SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd hh:mm:ss");
				SimpleDateFormat sdfShort = new SimpleDateFormat("yyyy/MM/dd");
				return sdf.parse(sdfShort.format(date) + " 23:59:59");
			}
		} catch (ParseException e) {
			e.printStackTrace();
		}
		return date;
	}
	
	//两个时间之间相差多少毫秒
	public static int getDatePoor(Date endDate, Date nowDate) {

		long nd = 1000 * 24 * 60 * 60;
		//		    long nh = 1000 * 60 * 60;
		//		    long nm = 1000 * 60;
		// long ns = 1000;
		// 获得两个时间的毫秒时间差异
		long diff = endDate.getTime() - nowDate.getTime();
		//		    // 计算差多少天
		long day = diff / nd;
		// 计算差多少小时
		//		    long hour = diff % nd / nh;
		// 计算差多少分钟
		//		    long min = diff % nd % nh / nm;
		// 计算差多少秒//输出结果
		// long sec = diff % nd % nh % nm / ns;
		return (int) day;
	}
		
	/**
	 * 获得两个时间的毫秒时间差异
	 * @param endDate 交大的时间
	 * @param nowDate 交小的时间
	 * @return
	 */
	public static int subFenZhong(Date startDate){
		//时间处理类
        try {
			SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
			long currentTime =System.currentTimeMillis();
			long startTime = df.parse(df.format(startDate)).getTime();//从对象中拿到时间
			long diff=(currentTime-startTime)/1000/60;
			System.out.println("当前系统时间为："+df.format(new Date())+"下单时间为："+df.format(startDate)+"两个时间差为："+diff+"分钟");
			//当前系统时间为：2020-05-19 18:40:17下单时间为：2020-05-19 16:40:00两个时间差为：120分钟
			return (int) diff;
		} catch (ParseException e) {
			e.printStackTrace();
		}
        return 0;
	}


	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		/*//这里会有一个异常，所以要用try catch捕获异常
		try {
			SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
			Date startDate  = sdf.parse("2020-05-19 16:40:0");
			int datePoor = subFenZhong(startDate);
			System.out.println(datePoor);
		}catch (Exception e){
		    e.printStackTrace();
		}*/
		
	}
	
	

}
